package com.gic.cloud.data.hook.api.service;

import com.gic.cloud.data.hook.api.dto.DownloadTask;
import com.gic.cloud.data.hook.api.dto.FlatQueryResult;
import com.gic.cloud.data.hook.api.entity.FlatQueryCondition;

import java.util.List;

/** 自助指标查询结果服务
 * @author Sam
 */
public interface IFlatQueryResultService {

	/** 自助指标查询
	 * @param tableId 指定的表明
	 * @param enterpriseIds 要查询的企业编号集合
	 * @param queryConditions 指定的查询条件
	 * @param orderField 指定的排序字段名
	 * @param orderDir 排序方向
	 * @param execDistinct 是否去重
	 * @param limitRange 取值范围，无限制则为 0（导出情况下）
	 * @return
	 */
	public FlatQueryResult getFlatQueryResult(String tableId, List<String> enterpriseIds, List<FlatQueryCondition> queryConditions, String orderField, String orderDir,
	                                          Boolean execDistinct, Integer queryDataType, Integer limitRange, int dataPermission);

	/** 评估自助指标查询下载任务数据量
	 * @param tableId
	 * @param enterpriseIds 要查询的企业编号集合
	 * @param queryConditions
	 * @param orderField
	 * @param orderDir
	 * @param execDistinct
	 * @return
	 */
	public Integer estimateFlatQueryDownloadTask(String tableId, List<String> enterpriseIds, List<FlatQueryCondition> queryConditions,
	                                             String orderField, String orderDir, Boolean execDistinct);

	/** 创建自助指标查询的下载任务
	 * @param userId
	 * @param name
	 * @param amount
	 * @param format
	 * @param tableId
	 * @param enterpriseIds 要查询的企业编号集合
	 * @param queryConditions
	 * @param orderField
	 * @param orderDir
	 * @param execDistinct
	 * @return
	 */
	public DownloadTask buildFlatQueryDownloadTask(String userId, String name, String moduleName, Integer amount, String format, String tableId, List<String> enterpriseIds, List<FlatQueryCondition> queryConditions,
	                                               String orderField, String orderDir, Boolean execDistinct,
	                                               String useCompress, Integer queryDataType, String applyId);

	/** 获取自助指标字段过滤条件数据
	 * @param tableId
	 * @param enterpriseIds 要查询的企业编号集合
	 * @param fieldName
	 * @return
	 */
	public List<Object> getFlatQueryFilterData(String tableId, List<String>enterpriseIds, String fieldName);


}
