package com.gic.cloud.data.hook.api.service;

import com.gic.cloud.common.api.base.Page;
import com.gic.cloud.data.hook.api.dto.FlatQueryFavo;
import com.gic.cloud.data.hook.api.dto.FlatQueryTable;
import com.gic.cloud.data.hook.api.dto.FlatQueryTableDetail;

/** 自助查询表对象服务
 * @author Sam.Z
 */
public interface IFlatQueryTableService {

	/** 获取指定编号的自助查询表信息
	 * @param userId 指定用户主键
	 * @param id 指定的自助查询表编号
	 * @return
	 */
	public FlatQueryTable queryFlatQueryTableById(String userId, String id);

	/** 请求自助查询表数据分页
	 * @param userId 指定用户主键
	 * @param fuzzy 模糊查询条件
	 * @param pageNum 指定的页数
	 * @param pageSize 分页数据数量
	 * @return
	 */
	public Page<FlatQueryTable> queryFlatQueryTablePage(String userId, String fuzzy, int pageNum, int pageSize);

	/** 获取指定自助查询表的详细信息
	 * @param userId 指定的用户主键
	 * @param tableId 指定的表主键
	 * @return
	 */
	public FlatQueryTableDetail queryFlatQueryTableDetailByTableId(String userId, String tableId);

	/** 获取指定自助查询的收藏
	 * @param userId 所属用户编号
	 * @param tableId 指定的表
	 * @return
	 */
	public FlatQueryFavo queryFlatQueryFavo(String userId, String tableId);

	/** 创建指定自助查询的收藏
	 * @param userId 所属用户编号
	 * @param tableId 指定的表
	 * @return
	 */
	public Integer createFlatQueryFavo(String userId, String tableId);

	/** 移除指定自助查询的收藏
	 * @param userId 所属用户编号
	 * @param tableId 指定的表
	 * @return
	 */
	public Integer removeFlatQueryFavo(String userId, String tableId);



}
