package com.gic.cloud.data.hook.service;

import cn.medubi.client.utils.LogPak;
import com.gic.cloud.data.hook.api.dto.FreeQuerySource;
import com.gic.cloud.data.hook.api.dto.FreeQueryTable;
import com.gic.cloud.data.hook.service.dao.FreeQuerySourceDao;
import com.gic.cloud.data.hook.service.entity.ConfigHelper;
import com.gic.cloud.data.hook.service.entity.MysqlSourceSwitch;
import org.springframework.beans.BeansException;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.ApplicationContext;
import org.springframework.context.ApplicationContextAware;
import org.springframework.stereotype.Component;

import javax.sql.DataSource;
import java.sql.*;
import java.util.ArrayList;
import java.util.List;
import java.util.Properties;

/** Mysql 辅助工具类
 * @author Sam
 */
@Component
public class MysqlHelper implements ApplicationContextAware {

	/** 日志类 */
	private static LogPak log = new LogPak(MysqlHelper.class);

	/** MySQL 数据源 */
	private static DataSource source = null;

	/** MySQL 内外网切换开关 */
	private static MysqlSourceSwitch switcher = null;

	/** 自定义查询数据源（已从系统库移除改为使用 db 团队指定库） */
	//@Autowired
	//private FreeQuerySourceDao freeQuerySourceDao;

	@Override
	public void setApplicationContext(ApplicationContext applicationContext) throws BeansException {
		log.debug("setApplicationContext", "准备初始化 MySQL 数据源");
		source = (DataSource)applicationContext.getBean("mysqlSource");
		log.debug("setApplicationContext", "准备初始化 MySQL 内外网开关");
		switcher = (MysqlSourceSwitch)applicationContext.getBean("mysqlSourceSwitch");
	}

	/** MysqlHelper 单例 */
	private static MysqlHelper instance = null;

	/** 构造函数 */
	public MysqlHelper() {
		instance = this;
	}

	/** 获取自定义查询数据源
	 * @param enterpriseId
	 * @return
	 */
	public FreeQuerySource getFreeQuerySource(String enterpriseId) {
		try {
			Connection conn = source.getConnection();
			String sql = "select * from tab_enterprisebyid_target where `enterprise_id`=?";
			PreparedStatement statment = conn.prepareStatement(sql);
			statment.setString(1, enterpriseId);
			ResultSet result = statment.executeQuery();
			while (result.next()) {
				FreeQuerySource source = new FreeQuerySource();
				source.setId(enterpriseId);
				source.setEnterpriseId(enterpriseId);
				source.setEnterpriseName(result.getString("enterprise_name"));
				String url = result.getString("url") + result.getString("options");
				if (switcher.getIsRunningOutsite()) {
					url = url.replace(switcher.getUrlReplaceTarget(), switcher.getUrlReplaceValue());
					log.debug("getFreeQuerySource", "外网环境使用，替换数据源为 " + url);
				} // IF OVER
				source.setUrl(url);
				source.setUsername(result.getString("usr"));
				source.setPassword(result.getString("password"));
				conn.close();
				return source;
			}
			conn.close();
		} catch (Exception e) {
			e.printStackTrace();
			log.debug("getFreeQuerySource", "获取编号为 " + enterpriseId + " 的商户自定义查询库连接失败");
		} // TRY CATCH OVER
		log.debug("getFreeQuerySource", "未能获取编号为 " + enterpriseId + " 的商户自定义查询库连接");
		return null;
	}

	/** 获取所有的自定义查询表信息
	 * @return
	 */
	public static List<FreeQueryTable> getFreeQueryTableList() {
		List<FreeQueryTable> tables = new ArrayList<FreeQueryTable>();
		try {
			Connection conn = source.getConnection();
			if (conn != null) {
				//String sql = "SELECT * FROM tab_enterprisebyid_extract_task WHERE `extract_type`=1 ORDER BY `priority_ranking`,`table_name`";
				String sql = "SELECT * FROM tab_selfdata_metastore ORDER BY `priority_ranking`,`table_name`";
				PreparedStatement statment = conn.prepareStatement(sql);
				ResultSet result = statment.executeQuery();
				while (result.next()) {
					FreeQueryTable table = new FreeQueryTable();
					Integer rank = result.getInt("priority_ranking");
					table.setRank(rank != null ? rank : 0);
					table.setTableId(result.getString("table_name"));
					table.setTableName(result.getString("table_describe"));
					tables.add(table);
				} // WHILE OVER
			}
			conn.close();
		} catch (Exception e) {
			e.printStackTrace();
			log.debug("getFreeQuerySource", "获取自定义查询用数据表信息失败");
		} // TRY CATCH OVER
		return tables;
	}


	/** 获取一个 FreeQuery 数据库的连接
	 * @return
	 */
	public static Connection getFreeQueryConnection(String enterpriseId) {
		Connection conn = null;
		log.debug("getFreeQueryConnection", "目标连接的企业编号为 " + enterpriseId);
		try {
//			String url = "jdbc:mysql://56cbb9f62fac6.sh.cdb.myqcloud.com:6634/dmwl_test";
//			Class.forName("com.mysql.jdbc.Driver");
//			conn = DriverManager.getConnection(url, "data_test", "iP7LwQsezaWW");
//			Properties prop = ConfigHelper.getPropertiesFile("data-hook-free-query-source");
//			String url = prop.getProperty("url");
//			String username = prop.getProperty("username");
//			String password = prop.getProperty("password");
//			conn = DriverManager.getConnection(url, username, password);
			//conn = source.getConnection(); // Spring 方式，已取消
			FreeQuerySource curSource = instance.getFreeQuerySource(enterpriseId); //instance.freeQuerySourceDao.getFreeQuerySourceByEnterpriseId(enterpriseId); // 已弃用从系统库取得自定义查询连接的方式
			conn = DriverManager.getConnection(curSource.getUrl(), curSource.getUsername(), curSource.getPassword());
			return conn;
		} catch (Exception ex) {
			ex.printStackTrace();
			if (conn != null) {
				try {
					conn.close();
				} catch (SQLException e) {
					e.printStackTrace();
				}
			}
			return null;
		} // TRY CATCH OVER
	}

}
