package com.gic.cloud.data.hook.service.entity;

import com.gic.cloud.data.hook.service.DecryptUtils;
import com.opencsv.ResultSetHelper;

import java.io.IOException;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Types;
import java.text.SimpleDateFormat;
import java.util.HashMap;
import java.util.List;

public class CsvResultSetHelper implements ResultSetHelper {

	/** 数据过滤模式 */
	private String filterMode = CsvDataFilterMode.NONE;

	/** 数据过滤用关键字集合 */
	private List<String> filters = null;

	public CsvResultSetHelper(String filterMode, List<String> filters) {
		this.filterMode = filterMode;
		this.filters = filters;
	}

	@Override
	public String[] getColumnNames(ResultSet resultSet) throws SQLException {
		String[] result = new String[0];
		int columnCount = resultSet.getMetaData().getColumnCount();
		if (columnCount > 0) {
			result = new String[columnCount];
			for (int i=0; i<columnCount; i++) {
				String rsColumnName = resultSet.getMetaData().getColumnLabel(i + 1);
				result[i] = rsColumnName;
			} // FOR OVER
		} // IF OVER
		return result;
	}

	@Override
	public String[] getColumnValues(ResultSet resultSet) throws SQLException, IOException {
		return this.getColumnValues(resultSet, false);
	}

	@Override
	public String[] getColumnValues(ResultSet resultSet, boolean b) throws SQLException, IOException {
		return this.getColumnValues(resultSet, false, "yyyy-MM-dd HH:mm:ss", "HH:mm:ss");
	}


	private static SimpleDateFormat datetimeFormatter = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");

	private static SimpleDateFormat dateFormatter = new SimpleDateFormat("yyyy-MM-dd");

	private static SimpleDateFormat timeFormatter = new SimpleDateFormat("hh:mm:ss");

	@Override
	public String[] getColumnValues(ResultSet resultSet, boolean b, String s, String s1) throws SQLException, IOException {
		String[] result = new String[0];
		int columnCount = resultSet.getMetaData().getColumnCount();
		if (columnCount > 0) {
			result = new String[columnCount];
			for (int i=0; i<columnCount; i++) {
				String columnName = resultSet.getMetaData().getColumnLabel(i + 1);
				// 数据处理判断
				boolean doDesensi = false; // 是否进行脱敏
				boolean doDecrypt = false; // 是否进行解密
				if (this.filterMode.equals(CsvDataFilterMode.DESENSI)) { // 如果需要脱敏
					if (this.filters != null && this.filters.size() > 0) {
						for (String filter : this.filters) {
							if (columnName.contains(filter)) {
								doDesensi = true;
								break;
							} // IF OVER
						} // FOR OVER
					} // IF OVER
				} else if (this.filterMode.equals(CsvDataFilterMode.DECRYPT)) { // 如果需要解密
					if (this.filters != null && this.filters.size() > 0) {
						for (String filter : this.filters) {
							if (columnName.equals(filter)) {
								doDecrypt = true;
								break;
							} // IF OVER
						} // FOR OVER
					} // IF OVER
				}
				// 数据处理
				if (doDesensi) { // 如果需要脱敏处理
					result[i] = "******";
				} else if (doDecrypt) { // 如果需要解密处理
					//System.out.println("CSV 解密字段名 " + columnName);
					String tmpResult = resultSet.getString(columnName);
					if (tmpResult != null && tmpResult.length() > 0) {
						//tmpResult = DecryptUtils.getInstance().decrypt(tmpResult);
						tmpResult = DecryptUtils.getInstance().decrypt(tmpResult);
					} // IF OVER
					result[i] = tmpResult;
				} else {
					int columnType = resultSet.getMetaData().getColumnType(i + 1);
					switch (columnType) {
						case Types.TIMESTAMP:
							result[i] = resultSet.getTimestamp(columnName) != null ? datetimeFormatter.format(resultSet.getTimestamp(columnName)) : "";
							break;
						case Types.DATE:
							//result[i] = resultSet.getTimestamp(columnName) != null ? dateFormatter.format(resultSet.getTimestamp(columnName)) : "";
							result[i] = resultSet.getDate(columnName) != null ? dateFormatter.format(resultSet.getDate(columnName)) : "";
							break;
						case Types.TIME:
							result[i] = resultSet.getTimestamp(columnName) != null ? timeFormatter.format(resultSet.getTimestamp(columnName)) : "";
							break;
						default:
							result[i] = String.valueOf(resultSet.getObject(columnName));
							break;
					} // SWITCH OVER
				} // IF ELSE OVER
			} // FOR OVER
		} // IF OVER
		return result;
	}
}
