package com.gic.cloud.data.hook.service.impl;

import com.alibaba.fastjson.JSON;
import com.gic.cloud.common.api.base.Page;
import com.gic.cloud.data.hook.api.dto.FlatQueryFavo;
import com.gic.cloud.data.hook.api.dto.FlatQueryField;
import com.gic.cloud.data.hook.api.dto.FlatQueryTable;
import com.gic.cloud.data.hook.api.dto.FlatQueryTableDetail;
import com.gic.cloud.data.hook.api.service.IFlatQueryTableService;
import com.gic.cloud.data.hook.service.dao.FlatQueryFavoDao;
import com.gic.cloud.data.hook.service.dao.FlatQueryFieldDao;
import com.gic.cloud.data.hook.service.dao.FlatQueryTableDao;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;

/** 自助取数表服务接口实现类
 * @author Sam.Z
 */
@Service
public class FlatQueryTableServiceImpl implements IFlatQueryTableService {

	private org.slf4j.Logger logger= LoggerFactory.getLogger(IFlatQueryTableService.class);

	/** 自助查询表信息 Dao */
	@Autowired
	private FlatQueryTableDao flatQueryTableDao;

	/** 自助查询收藏 Dao */
	@Autowired
	private FlatQueryFavoDao flatQueryFavoDao;

	/** 自助查询字段  DAO */
	@Autowired
	private FlatQueryFieldDao flatQueryFieldDao;

	@Override
	public FlatQueryTable queryFlatQueryTableById(String userId, String id) {
		FlatQueryTable table = this.flatQueryTableDao.getFlatQueryTableById(userId, id);
		logger.debug("json 格式为 " + JSON.toJSONString(table));
		return table;
	}

	@Override
	public Page<FlatQueryTable> queryFlatQueryTablePage(String userId, String fuzzy, int pageNum, int pageSize) {
		Page<FlatQueryTable> result = new Page<>();
		PageHelper.startPage(pageNum, pageSize); // 设置分页参数
		List<FlatQueryTable> query = flatQueryTableDao.getFlatQueryTable(userId, fuzzy);
		// 生成数据输出信息
		PageInfo<FlatQueryTable> preResult = new PageInfo<>(query);
		result.setList(preResult.getList()); // 设置数据集
		result.setTotal(preResult.getTotal()); // 设置总数
		result.setPageNum(preResult.getPageNum()); // 设置当前页数
		logger.debug("json 格式为 " + JSON.toJSONString(result));
		return result;
	}

	@Override
	public FlatQueryTableDetail queryFlatQueryTableDetailByTableId(String userId, String tableId) {
		FlatQueryTableDetail result = new FlatQueryTableDetail();
		FlatQueryTable table = this.flatQueryTableDao.getFlatQueryTableById(userId, tableId);
		if (table != null) {
			// 增加表格查询量
			table.setClickCount(table.getClickCount() + 1);
			this.flatQueryTableDao.updateFlatQueryTable(table);
			// 返回结果集
			result.setTableId(table.getTableId());
			result.setTableName(table.getName());
			List<FlatQueryField> fields = this.flatQueryFieldDao.getFlatQueryFieldByTableId(tableId);
			result.setFieldDefines(fields);
		} // IF OVER
		return result;
	}

	/** 获取指定自助查询的收藏
	 * @param userId 所属用户编号
	 * @param tableId 指定的表
	 * @return
	 */
	public FlatQueryFavo queryFlatQueryFavo(String userId, String tableId) {
		return this.flatQueryFavoDao.getFlatQueryFavo(userId, tableId);
	}

	/** 创建指定自助查询的收藏
	 * @param userId 所属用户编号
	 * @param tableId 指定的表
	 * @return
	 */
	public Integer createFlatQueryFavo(String userId, String tableId) {
		return this.flatQueryFavoDao.addFlatQueryFavo(userId + tableId, userId, tableId);
	}

	/** 移除指定自助查询的收藏
	 * @param userId 所属用户编号
	 * @param tableId 指定的表
	 * @return
	 */
	public Integer removeFlatQueryFavo(String userId, String tableId) {
		return this.flatQueryFavoDao.removeFlatQueryFavo(userId, tableId);
	}



}
