package com.gic.cloud.data.hook.web;

import com.gic.cloud.common.api.base.Page;
import com.gic.cloud.data.hook.api.dto.*;
import com.gic.cloud.data.hook.api.entity.FlatQueryExecuteRequest;
import com.gic.cloud.data.hook.api.entity.GeneralResult;
import com.gic.cloud.data.hook.api.service.IFlatQueryResultService;
import com.gic.cloud.data.hook.api.service.IFreeQueryService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.Date;
import java.util.List;

/** 自定义查询相关 Controller
 * @author Sam
 */
@Controller
public class FreeQueryController {

	/** 自定义查询服务 */
	@SuppressWarnings("SpringJavaInjectionPointsAutowiringInspection")
	@Autowired
	private IFreeQueryService freeQueryService;

	/** 获取所有的自定义查询表信息
	 * @param request
	 * @param response
	 * @return
	 */
	@RequestMapping("/free-query-table-list")
	public List<FreeQueryTable> freeQueryTableList(HttpServletRequest request, HttpServletResponse response) {
		return this.freeQueryService.getFreeQueryTableList();
	}

	/** 获取所有的自定义查询记录
	 * @param userId 指定的用户编号
	 * @param fuzzy 模糊查询条件
	 * @param request
	 * @param response
	 * @return
	 */
	@RequestMapping("/free-query-record-list")
	public List<FreeQueryRecord> freeQueryRecordList(String userId, String fuzzy, HttpServletRequest request, HttpServletResponse response) {
		return this.freeQueryService.getFreeQueryRecordListByUserId(userId, fuzzy); // 限定 10 个/分页
	}

	/** 创建自定义查询记录
	 * @param userId 指定的用户编号
	 * @param name 查询记录名称
	 * @param content 查询脚本内容
	 * @param request
	 * @param response
	 * @return
	 */
	@RequestMapping("/create-free-query-record")
	public List<FreeQueryRecord> createFreeQueryRecord(String recordId, String userId, String name, String content, HttpServletRequest request, HttpServletResponse response) {
		FreeQueryRecord record = new FreeQueryRecord();
		record.setId(recordId);
		record.setUserId(userId);
		record.setName(name);
		record.setContent(content);
		record.setUpdateTime(new Date());
		this.freeQueryService.createFreeQueryRecord(record);
		return this.freeQueryService.getFreeQueryRecordListByUserId(userId, ""); // 返回所有集合记录
	}

	/** 更新自定义查询记录
	 * @param userId 指定的用户编号
	 * @param recordId 要更新的记录编号
	 * @param name 更新的记录名称
	 * @param content 更新的记录脚本内容
	 * @param request
	 * @param response
	 * @return
	 */
	@RequestMapping("/update-free-query-record")
	public List<FreeQueryRecord> updateFreeQueryRecord(String userId, String recordId, String name, String content, HttpServletRequest request, HttpServletResponse response) {
		FreeQueryRecord preRecord = this.freeQueryService.getFreeQueryRecordByUserIdAndRecordId(userId, recordId);
		if (preRecord != null) {
			preRecord.setName(name);
			preRecord.setContent(content);
			preRecord.setUpdateTime(new Date());
			this.freeQueryService.updateFreeQueryRecord(preRecord);
		} // IF OVER
		return this.freeQueryService.getFreeQueryRecordListByUserId(userId, ""); // 返回所有集合记录
	}

	/** 删除指定的自定义查询记录
	 * @param userId 指定的用户编号
	 * @param recordId 指定的记录编号
	 * @param request
	 * @param response
	 * @return
	 */
	@RequestMapping("/delete-free-query-record")
	public List<FreeQueryRecord> deleteFreeQueryRecord(String userId, String recordId, HttpServletRequest request, HttpServletResponse response) {
		FreeQueryRecord preRecord = this.freeQueryService.getFreeQueryRecordByUserIdAndRecordId(userId, recordId);
		if (preRecord != null) {
			this.freeQueryService.deleteFreeQueryRecord(recordId);
		} // IF OVER
		return this.freeQueryService.getFreeQueryRecordListByUserId(userId, ""); // 返回所有集合记录
	}

	/** 获取自定义查询总数
	 * @param sql
	 * @param request
	 * @param response
	 * @return
	 */
	@RequestMapping("/get-free-query-count")
	public GeneralResult getFreeQueryCount(String sql, String enterpriseId, HttpServletRequest request, HttpServletResponse response) {
		GeneralResult result = new GeneralResult();
		result.setInfo(String.valueOf(this.freeQueryService.getFreeQueryCount(sql, enterpriseId)));
		return result;
	}

	/** 获取自定义查询结果
	 * @param sql
	 * @param request
	 * @param response
	 * @return
	 */
	@RequestMapping("/get-free-query-result")
	public FreeQueryResult getFreeQueryResult(String sql, String enterpriseId, Integer queryDataType, Integer desensiType, HttpServletRequest request, HttpServletResponse response) {
		return this.freeQueryService.getFreeQueryResult( "SELECT tmpTable.* FROM (" + sql + ") AS tmpTable LIMIT 1000", enterpriseId, queryDataType, desensiType);
	}


	/** 生成自助查询下载任务
	 * @param userId
	 * @param name
	 * @param amount
	 * @param format
	 * @param sql
	 * @param request
	 * @param response
	 * @return
	 */
	@RequestMapping("/build-free-query-download-task")
	public DownloadTask buildDownloadTask(String userId, String name, Integer amount, String format, String sql, String enterpriseId, Boolean useCompress, Integer dataType, String applyId, Integer desensiType, HttpServletRequest request, HttpServletResponse response) {
		return this.freeQueryService.buildFreeQueryDownloadTask(userId, name, amount, format, sql, enterpriseId, useCompress, dataType, applyId, desensiType);
	}

	/** 获取指定表明的所有字段
	 * @param tableName
	 * @param request
	 * @param response
	 * @return
	 */
	@RequestMapping("/get-columns-by-table")
	public List<String> getColumnsByTable(String tableName, HttpServletRequest request, HttpServletResponse response) {
		return this.freeQueryService.getColumnsByTable(tableName);
	}



}
